@@ -51,6 +51,7 @@
    work right without it.  Maybe a bug in Devel::PPPort?  */
 #define NEED_sv_2pv_flags
 #include "ppport.h"
+    /* Defines PERL_REVISION, etc. (if perl.h doesn't) */
 
 #ifndef C_PANELFUNCTION
 #  define PANEL int
@@ -72,10 +73,10 @@
    defined in Perl's CORE/pp.h via our inclusion of perl.h above.
 */
 
-/* What a mess. :( */
-#ifndef PERL_VERSION
-#  define PL_sv_undef  sv_undef
-#  define PL_sv_yes    sv_yes
+#if PERL_REVISION > 5 || (PERL_REVISION == 5 && PERL_VERSION >= 16)
+  #define HAVE_PERL_UTF8 1
+#else
+  #define HAVE_PERL_UTF8 0
 #endif
 
 /*
@@ -98,8 +99,8 @@ int base;
     case 2:  c_win = 0; c_x = 1; c_arg = 2; break;
     case 3:  c_win = 1; c_x = 2; c_arg = 3; break;
     default:
-	croak("Curses function '%s' called with too %s arguments", fn,
-	      nargs < base ? "few" : "many");
+    croak("Curses function '%s' called with too %s arguments", fn,
+          nargs < base ? "few" : "many");
     }
     c_function = fn;
 }
@@ -111,8 +112,8 @@ int nargs;
 int base;
 {
     if (nargs != base)
-	croak("Curses function '%s' called with too %s arguments", fn,
-	      nargs < base ? "few" : "many" );
+    croak("Curses function '%s' called with too %s arguments", fn,
+          nargs < base ? "few" : "many" );
 
     c_function = fn;
 }
@@ -172,10 +173,10 @@ chtype ch;
 {
     if (ch == ERR || ch > 255) { sv_setiv(sv, (I32)ch); }
     else {
-	char tmp[2];
-	tmp[0] = (char)ch;
-	tmp[1] = (char)0;
-	sv_setpv(sv, tmp);
+    char tmp[2];
+    tmp[0] = (char)ch;
+    tmp[1] = (char)0;
+    sv_setpv(sv, tmp);
     }
 }
 
@@ -185,12 +186,12 @@ SV *sv;
 int argnum;
 {
     if (sv_derived_from(sv, "Curses::Field"))
-	return (FIELD *)SvIV((SV*)SvRV(sv));
+    return (FIELD *)SvIV((SV*)SvRV(sv));
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses field",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses field",
+          argnum, c_function);
     else
-	croak("argument is not a Curses field");
+    croak("argument is not a Curses field");
 }
 
 static void
@@ -210,12 +211,12 @@ SV *sv;
 int argnum;
 {
     if (sv_derived_from(sv, "Curses::Form"))
-	return (FORM *)SvIV((SV*)SvRV(sv));
+    return (FORM *)SvIV((SV*)SvRV(sv));
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses form",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses form",
+          argnum, c_function);
     else
-	croak("argument is not a Curses form");
+    croak("argument is not a Curses form");
 }
 
 static void
@@ -232,12 +233,12 @@ SV *sv;
 int argnum;
 {
     if (sv_derived_from(sv, "Curses::Item"))
-	return (ITEM *)SvIV((SV*)SvRV(sv));
+    return (ITEM *)SvIV((SV*)SvRV(sv));
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses item",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses item",
+          argnum, c_function);
     else
-	croak("argument is not a Curses item");
+    croak("argument is not a Curses item");
 }
 
 
@@ -255,12 +256,12 @@ SV *sv;
 int argnum;
 {
     if (sv_derived_from(sv, "Curses::Menu"))
-	return (MENU *)SvIV((SV*)SvRV(sv));
+    return (MENU *)SvIV((SV*)SvRV(sv));
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses menu",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses menu",
+          argnum, c_function);
     else
-	croak("argument is not a Curses menu");
+    croak("argument is not a Curses menu");
 }
 
 static void
@@ -277,12 +278,12 @@ SV *sv;
 int argnum;
 {
     if (sv_derived_from(sv, "Curses::Panel"))
-	return (PANEL *)SvIV((SV*)SvRV(sv));
+    return (PANEL *)SvIV((SV*)SvRV(sv));
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses panel",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses panel",
+          argnum, c_function);
     else
-	croak("argument is not a Curses panel");
+    croak("argument is not a Curses panel");
 }
 
 static void
@@ -299,12 +300,12 @@ SV *sv;
 int argnum;
 {
     if (sv_derived_from(sv, "Curses::Screen"))
-	return (SCREEN *)SvIV((SV*)SvRV(sv));
+    return (SCREEN *)SvIV((SV*)SvRV(sv));
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses screen",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses screen",
+          argnum, c_function);
     else
-	croak("argument is not a Curses screen");
+    croak("argument is not a Curses screen");
 }
 
 static void
@@ -325,10 +326,10 @@ int argnum;
       return ret;
     }
     if (argnum >= 0)
-	croak("argument %d to Curses function '%s' is not a Curses window",
-	      argnum, c_function);
+    croak("argument %d to Curses function '%s' is not a Curses window",
+          argnum, c_function);
     else
-	croak("argument is not a Curses window");
+    croak("argument is not a Curses window");
 }
 
 static void
@@ -350,9 +351,9 @@ char *name;
     if (len > 0) {
         name[len - 1] = 0;
 
-	SvCUR(sv) = strlen(name);
-	SvPOK_only(sv);
-	*SvEND(sv) = 0;
+    SvCUR(sv) = strlen(name);
+    SvPOK_only(sv);
+    *SvEND(sv) = 0;
     }
 }
 
@@ -368,11 +369,11 @@ chtype *name;
     if (len - len % rs > rs) {            /* find even multiple of rs */
         name[len - len % rs - rs] = 0;
 
-	while (*name++) { n++; }
+    while (*name++) { n++; }
 
-	SvCUR(sv) = n;
-	SvPOK_only(sv);
-	*(chtype *)SvEND(sv) = 0;
+    SvCUR(sv) = n;
+    SvPOK_only(sv);
+    *(chtype *)SvEND(sv) = 0;
     }
 }
 
@@ -385,182 +386,12 @@ SV *sv;
 }
 
 
-/* Combined Normal/Wide-Character helper functions */
-#include <wchar.h>
-
-/* Set SV to a one-character (not -byte!) Perl string holding a given wide character */
-static void c_wchar2sv(SV* sv, wchar_t wc) {
-	if (wc <= 0xff) {
-		char s[] = { wc, 0 };
-		sv_setpv(sv, s);
-		SvPOK_on(sv);
-		SvUTF8_off(sv);
-	} else {
-		char s[UTF8_MAXBYTES + 1] = { 0 };
-		char *s_end = (char *)uvchr_to_utf8((U8 *)s, wc);
-		*s_end = 0;
-		sv_setpv(sv, s);
-		SvPOK_on(sv);
-		SvUTF8_on(sv);
-	}
-}
-
-/* Set SV to a Perl string holding a given byte string */
-static void c_bstr2sv(SV *sv, unsigned char *bs) {
-	SvPOK_on(sv);
-	sv_setpv(sv, (char *)bs);
-	SvUTF8_off(sv);
-}
-
-/* Set SV to a Perl string holding a given wide string */
-static void c_wstr2sv(SV *sv, wchar_t *ws) {
-	wint_t *ws_p;
-	int need_utf8 = 0;
-	size_t ws_len = wcslen(ws);
-	
-	for (ws_p = ws; *ws_p; ws_p++) {
-		if (*ws_p > 0xff) {
-			need_utf8 = 1;
-			break;
-		}
-	}
-	SvPOK_on(sv);
-	if (need_utf8) {
-		U8 *u8, *u8_p;
-		u8 = (U8 *)sv_grow(sv, (ws_len + 1) * UTF8_MAXBYTES);
-		for (ws_p = ws, u8_p = u8; *ws_p; ws_p++)
-			u8_p = uvchr_to_utf8(u8_p, *ws_p);
-		*u8_p = 0;
-		SvCUR_set(sv, u8_p - u8);
-		SvUTF8_on(sv);
-	} else {
-		U8 *u8, *u8_p;
-		u8 = (U8 *)sv_grow(sv, ws_len + 1);
-		for (ws_p = ws, u8_p = u8; *ws_p; ws_p++, u8_p++)
-			*u8_p = *ws_p;
-		*u8_p = 0;
-		SvCUR_set(sv, ws_len);
-		SvUTF8_off(sv);
-	}
-}
-
-/* Extract a wide character from a SV holding a one-character Perl string */
-/* Fails (returning WEOF) if SV doesn't hold a string or the string is not one character long */
-static wint_t c_sv2wchar(SV *sv) {
-	U8 *s;
-	STRLEN s_len;
-	if (!SvPOK(sv))
-		return WEOF;
-	s = (U8 *)SvPV(sv, s_len);
-	if (s_len == 0)
-		return WEOF;
-	if (SvUTF8(sv)) {
-		STRLEN len;
-		UV uv = utf8_to_uvchr_buf(s, s + s_len, &len);
-		if (len != s_len)
-			return WEOF;
-		return (wint_t) uv;
-	} else {
-		if (s_len != 1)
-			return WEOF;
-		return *s;
-	}
-}
-
-/* Extract a char (byte) string from a SV holding a Perl string */
-/* Fails (returning NULL) if SV doesn't hold a string or the string has characters not fitting into a byte or doesn't UTF-8 decode */
-/* b_len set to length of result */
-/* Caller must free() result if need_free is set */
-static unsigned char *c_sv2bstr(SV *sv, size_t *b_len, int *need_free) {
-	U8 *s, *s_p, *s_end;
-	STRLEN s_len;
-	unsigned char *bs, *bs_p;
-
-	if (!SvPOK(sv)) {
-		*need_free = 0;
-		return NULL;
-	}
-	s = (U8 *)SvPV(sv, s_len);
-	s_p = s;
-	s_end = s + s_len;
-	if (SvUTF8(sv)) {
-		bs = malloc(s_len + 1); /* number of bytes is an upper bound on the number of characters */
-		if (bs == NULL) croak("c_sv2bstr: malloc");
-		bs_p = bs;
-		while (s_p < s_end) {
-			if (UTF8_IS_INVARIANT(*s_p)) {
-				*bs_p++ = *s_p++;
-			} else {
-				STRLEN len;
-				UV uv = utf8_to_uvchr_buf(s_p, s_end, &len);
-				if (uv > 0xff) {
-					*need_free = 0;
-					*b_len = 0;
-					return NULL;
-				}
-				*bs_p++ = uv;
-				s_p += len;
-			}
-		}
-		if (s_p != s_end) {
-			*need_free = 0;
-			*b_len = 0;
-			return NULL;
-		}
-		*bs_p = 0;
-		*b_len = s_len;
-		*need_free = 1;
-		return bs;
-	} else {
-		*need_free = 0;
-		*b_len = s_len;
-		return (unsigned char *)s;
-	}
-
-}
-
-/* Extract a wide char string from a SV holding a Perl string */
-/* Fails (returning NULL) if SV doesn't hold a string or doesn't UTF-8 decode */
-/* w_len set to length of result */
-/* Caller must free() result */
-static wint_t *c_sv2wstr(SV *sv, size_t *w_len) {
-	U8 *s, *s_p, *s_end;
-	STRLEN s_len;
-	wint_t *ws, *ws_p;
-
-	if (!SvPOK(sv)) return NULL;
-	s = (U8 *)SvPV(sv, s_len);
-	s_p = s;
-	s_end = s + s_len;
-	ws = malloc((s_len + 1) * sizeof(*ws)); /* number of bytes is an upper bound on the number of characters */
-	if (ws == NULL) croak("c_sv2wstr: malloc");
-	ws_p = ws;
-	if (SvUTF8(sv)) {
-		while (s_p < s_end) {
-			if (UTF8_IS_INVARIANT(*s_p)) {
-				*ws_p++ = *s_p++;
-			} else {
-				STRLEN len;
-				*ws_p++ = utf8_to_uvchr_buf(s_p, s_end, &len);
-				s_p += len;
-			}
-		}
-		if (s_p != s_end) {
-			free(ws);
-			*w_len = 0;
-			return NULL;
-		}
-	} else {
-		s_p = s;
-		while (s_p < s_end) {
-			*ws_p++ = *s_p++;
-		}
-	}
-	*ws_p = 0;
-	*w_len = s_len;
-	return ws;
-}
-
+#if HAVE_PERL_UTF8
+  #include "CursesWide.c"
+  #define HAVE_WIDE_SV_HELPER 1
+#else
+  #define HAVE_WIDE_SV_HELPER 0
+#endif
 
 /*
 **  Cheesy, I know.  But it works.
@@ -568,6 +399,12 @@ static wint_t *c_sv2wstr(SV *sv, size_t *w_len) {
 
 
 #include "CursesFun.c"
+#if HAVE_WIDE_SV_HELPER
+  #include "CursesFunWide.c"
+  #define HAVE_WIDE_XS_FUNCTIONS 1
+#else
+  #define HAVE_WIDE_XS_FUNCTIONS 0
+#endif
 #include "CursesVar.c"
 #include "CursesCon.c"
 #include "CursesBoot.c"
